@@ -1,7 +1,7 @@
 /**
  * @file db.c sqlite backend
  * 
- * Copyright (C) 2007-2012  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2012  Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -231,10 +231,10 @@ db_vacuum (void)
 static void
 db_open (void)
 {
-	gchar		*filename;
-	gint		res;
+	gchar	*filename;
+	gint	res;
 
-	filename = common_create_cache_filename (NULL, "liferea", "db");
+	filename = common_create_data_filename ("liferea.db");
 	debug1 (DEBUG_DB, "Opening DB file %s...", filename);
 	res = sqlite3_open_v2 (filename, &db, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL);
 	if (SQLITE_OK != res)
@@ -255,7 +255,6 @@ void
 db_init (void)
 {
 	gint		res;
-	GError          *error;
 		
 	debug_enter ("db_init");
 
@@ -552,18 +551,22 @@ db_init (void)
 	         "   DELETE FROM items WHERE item_id IN (SELECT id FROM tmp_id LIMIT 1000);"
 	         "   DROP TABLE tmp_id;"
 		 "END;");
-
- 	debug0 (DEBUG_DB, "Checking for search folder items without a feed list node...\n");
-	db_exec ("DELETE FROM search_folder_items WHERE parent_node_id NOT IN "
-	         "(SELECT node_id FROM node);");
         
+	debug0 (DEBUG_DB, "Checking for search folder items without a feed list node...\n");
+	db_exec ("DELETE FROM search_folder_items WHERE parent_node_id NOT IN "
+        	 "(SELECT node_id FROM node);");
+
 	debug0 (DEBUG_DB, "Checking for search folder items without a search folder...\n");
 	db_exec ("DELETE FROM search_folder_items WHERE node_id NOT IN "
-         	 "(SELECT node_id FROM node);");
+        	 "(SELECT node_id FROM node);");
 
 	debug0 (DEBUG_DB, "Checking for search folder with comments...\n");
 	db_exec ("DELETE FROM search_folder_items WHERE comment = 1;");
-			  
+
+	debug0 (DEBUG_DB, "Checking for subscription metadata without node...\n");
+	db_exec ("DELETE FROM subscription_metadata WHERE node_id NOT IN "
+          	 "(SELECT node_id FROM node);");
+
 	debug0 (DEBUG_DB, "DB cleanup finished. Continuing startup.");
 		
 	/* 4. Creating triggers (after cleanup so it is not slowed down by triggers) */
@@ -727,22 +730,16 @@ db_init (void)
 	debug_exit ("db_init");
 }
 
-static void
-db_free_statements (gpointer key, gpointer value, gpointer user_data)
-{
-	value = NULL;
-}
-
 void
 db_deinit (void) 
 {
+
 	debug_enter ("db_deinit");
 	
 	if (FALSE == sqlite3_get_autocommit (db))
 		g_warning ("Fatal: DB not in auto-commit mode. This is a bug. Data may be lost!");
 	
 	if (statements) {
-		g_hash_table_foreach (statements, db_free_statements, NULL);
 		g_hash_table_destroy (statements);	
 		statements = NULL;
 	}
@@ -969,17 +966,16 @@ db_item_search_folders_update (itemPtr item)
 	iter = list = vfolder_get_all_with_item_id (item);
 	while (iter) {
 		vfolderPtr vfolder = (vfolderPtr)iter->data;
-
 		sqlite3_reset (stmt);
 		sqlite3_bind_text (stmt, 1, vfolder->node->id, -1, SQLITE_TRANSIENT);
 		sqlite3_bind_text (stmt, 2, item->nodeId, -1, SQLITE_TRANSIENT);
 		sqlite3_bind_int (stmt, 3, item->id);
- 		res = sqlite3_step (stmt);
+		res = sqlite3_step (stmt);
 
 		if (SQLITE_DONE != res) 
 			g_warning ("item add to search folder failed (error code=%d, %s)", res, sqlite3_errmsg (db));
-
 		iter = g_slist_next (iter);
+
 	}
 	g_slist_free (list);
 
@@ -992,20 +988,19 @@ db_item_search_folders_update (itemPtr item)
 	iter = list = vfolder_get_all_without_item_id (item);
 	while (iter) {
 		vfolderPtr vfolder = (vfolderPtr)iter->data;
+		sqlite3_reset (stmt);
+		sqlite3_bind_text (stmt, 1, vfolder->node->id, -1, SQLITE_TRANSIENT);
+		sqlite3_bind_int (stmt, 2, item->id);
+		res = sqlite3_step (stmt);
 
- 		sqlite3_reset (stmt);
- 		sqlite3_bind_text (stmt, 1, vfolder->node->id, -1, SQLITE_TRANSIENT);
- 		sqlite3_bind_int (stmt, 2, item->id);
- 		res = sqlite3_step (stmt);
- 
- 		if (SQLITE_DONE != res) 
+		if (SQLITE_DONE != res) 
 			g_warning ("item remove from search folder failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+		iter = g_slist_next (iter);
 
- 		iter = g_slist_next (iter);
- 	}
+	}
 	g_slist_free (list);
- 
- 	sqlite3_finalize (stmt);
+
+	sqlite3_finalize (stmt);
 }
 
 void
@@ -1358,7 +1353,7 @@ db_search_folder_load (const gchar *id)
 	stmt = db_get_statement ("searchFolderLoadStmt");
 	res = sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
 	if (SQLITE_OK != res)
-		g_error ("db_load_metadata: sqlite bind failed (error code %d)!", res);
+		g_error ("db_search_folder_load: sqlite bind failed (error code %d)!", res);
 	
 	itemSet = g_new0 (struct itemSet, 1);
 	itemSet->nodeId = (gchar *)id;
@@ -1451,7 +1446,7 @@ db_search_folder_get_item_count (const gchar *id)
 }
 
 static GSList *
-db_subscription_metadata_load(const gchar *id) 
+db_subscription_metadata_load (const gchar *id) 
 {
 	GSList		*metadata = NULL;
 	sqlite3_stmt	*stmt;
@@ -1460,7 +1455,7 @@ db_subscription_metadata_load(const gchar *id)
 	stmt = db_get_statement ("subscriptionMetadataLoadStmt");
 	res = sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
 	if (SQLITE_OK != res)
-		g_error ("db_load_metadata: sqlite bind failed (error code %d)!", res);
+		g_error ("db_subscription_metadata_load: sqlite bind failed (error code %d)!", res);
 
 	while (sqlite3_step (stmt) == SQLITE_ROW) {
 		metadata = db_metadata_list_append (metadata, sqlite3_column_text(stmt, 0), 
@@ -1489,10 +1484,9 @@ db_subscription_metadata_update_cb (const gchar *key,
 	sqlite3_bind_text (stmt, 4, value, -1, SQLITE_TRANSIENT);
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res) 
-		g_warning ("Update in \"metadata\" table failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+		g_warning ("Update in \"subscription_metadata\" table failed (error code=%d, %s)", res, sqlite3_errmsg (db));
 
 	sqlite3_finalize (stmt);
-
 }
 
 static void
@@ -1530,13 +1524,13 @@ db_subscription_update (subscriptionPtr subscription)
 	
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res)
-		g_warning ("Could not update subscription info %s in DB (error code %d)!", subscription->node->id, res);
-
+		g_warning ("Could not update subscription info for node id %s in DB (error code %d)!", subscription->node->id, res);
+	
 	sqlite3_finalize (stmt);
 
 	db_subscription_metadata_update (subscription);
 		
-	debug_end_measurement (DEBUG_DB, "subscription_update");
+	debug_end_measurement (DEBUG_DB, "subscription update");
 }
 
 void
@@ -1581,11 +1575,11 @@ db_node_update (nodePtr node)
 	
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res)
-		g_warning ("Could not update subscription info %s in DB (error code %d)!", node->id, res);
+		g_warning ("Could not update node info %s in DB (error code %d)!", node->id, res);
 
 	sqlite3_finalize (stmt);
-
-	debug_end_measurement (DEBUG_DB, "subscription_update");
+		
+	debug_end_measurement (DEBUG_DB, "node update");
 }
 
 static gboolean
@@ -1626,7 +1620,6 @@ void
 db_node_cleanup (nodePtr root)
 {
 	sqlite3_stmt	*stmt;
-	gint		res;
 
 	debug0 (DEBUG_DB, "Cleaning node ids...");
 
