@@ -1,7 +1,7 @@
 /**
  * @file ttrss_source.c  tt-rss feed list source support
  * 
- * Copyright (C) 2010-2012 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2010-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,25 +34,21 @@
 #include "node.h"
 #include "subscription.h"
 #include "update.h"
-#include "ui/auth_dialog.h"
 #include "ui/liferea_dialog.h"
-#include "ui/ui_common.h"
 #include "fl_sources/node_source.h"
 #include "fl_sources/opml_source.h"
 
-// FIXME: Avoid doing requests when we are not logged in yet!
-
-/** default tt-rss subscription list update interval = once a day */
-#define TTRSS_SOURCE_UPDATE_INTERVAL 60*60*24
-
-/** create a tt-rss source with given node as root */ 
+/** Initialize a TinyTinyRSS source with given node as root */ 
 static ttrssSourcePtr
 ttrss_source_new (nodePtr node) 
 {
 	ttrssSourcePtr source = g_new0 (struct ttrssSource, 1) ;
-	source->root = node; 
+	source->root = node;
+	source->apiLevel = 0;
 	source->actionQueue = g_queue_new (); 
-	source->loginState = TTRSS_SOURCE_STATE_NONE; 
+	source->loginState = TTRSS_SOURCE_STATE_NONE;
+	source->categories = g_hash_table_new (g_direct_hash, g_direct_equal);
+	source->categoryNodes = g_hash_table_new (g_direct_hash, g_direct_equal);
 	
 	return source;
 }
@@ -76,7 +72,7 @@ ttrss_source_get_config_cb (const struct updateResult * const result, gpointer u
 	ttrssSourcePtr	source = (ttrssSourcePtr) userdata;
 	subscriptionPtr subscription = source->root->subscription;
 
-	debug1 (DEBUG_UPDATE, "tt-rss getConfig processing... >>>%s<<<", result->data);
+	debug1 (DEBUG_UPDATE, "TinyTinyRSS getConfig processing... >>>%s<<<", result->data);
 
 	/* We expect something like:
 
@@ -92,9 +88,10 @@ ttrss_source_get_config_cb (const struct updateResult * const result, gpointer u
 			const gchar *result = json_get_string (node, "daemon_is_running");
 			if (result && g_str_equal ("true", result)) {
 				source->selfUpdating = TRUE;
-				debug0 (DEBUG_UPDATE, "This tt-rss source is self-updating!");
+				debug0 (DEBUG_UPDATE, "This TinyTinyRSS source is self-updating!");
 			} else {
-				debug0 (DEBUG_UPDATE, "This tt-rss source is not self-updating!");
+				ui_show_info_box (_("This TinyTinyRSS source is not self-updating and Liferea does not support updating TinyTinyRSS remotely at the moment. Please change the TinyTinyRSS preferences to automatic updating!"));
+				debug0 (DEBUG_UPDATE, "This TinyTinyRSS source is not self-updating!");
 			}
 
 			g_object_unref (parser);
@@ -108,52 +105,81 @@ ttrss_source_get_config_cb (const struct updateResult * const result, gpointer u
 		subscription_update (subscription, flags);
 }
 
+static void
+ttrss_source_set_login_error (ttrssSourcePtr source, gchar *msg)
+{
+	g_free (source->root->subscription->updateError);
+	source->root->subscription->updateError = msg;
+	source->root->available = FALSE;
+
+	g_warning ("TinyTinyRSS login failed: error '%s'!\n", msg);
+
+	if (++source->authFailures < TTRSS_SOURCE_MAX_AUTH_FAILURES)
+		source->loginState = TTRSS_SOURCE_STATE_NONE;
+	else
+		source->loginState = TTRSS_SOURCE_STATE_NO_AUTH;
+}
+
 static void
 ttrss_source_login_cb (const struct updateResult * const result, gpointer userdata, updateFlags flags)
 {
 	ttrssSourcePtr	source = (ttrssSourcePtr) userdata;
 	subscriptionPtr subscription = source->root->subscription;
-		
-	debug1 (DEBUG_UPDATE, "tt-rss login processing... >>>%s<<<", result->data);
+	JsonParser	*parser;
+	
+	debug1 (DEBUG_UPDATE, "TinyTinyRSS login processing... >>>%s<<<", result->data);
 	
 	g_assert (!source->session_id);
 	
-	if (result->data && result->httpstatus == 200) {
-		JsonParser *parser = json_parser_new ();
-		
-		if (json_parser_load_from_data (parser, result->data, -1, NULL)) {
-			JsonNode *node = json_parser_get_root (parser);
-
-			if (json_get_string (node, "error"))
-				g_warning ("tt-rss login failed: error '%s'!\n", json_get_string (node, "error"));
-			
-			source->session_id = g_strdup (json_get_string (json_get_node (node, "content"), "session_id"));
-			if (source->session_id) {
-				debug1 (DEBUG_UPDATE, "Found session_id: >>>%s<<<!", source->session_id);
-			} else {
-				g_warning ("No tt-rss session_id found in response!\n%s", result->data);
-			}
-			
-			g_object_unref (parser);
-		} else {
-			g_warning ("Invalid JSON returned on tt-rss login! >>>%s<<<", result->data);
-		}
-	} else {
-		g_warning ("tt-rss login failed: HTTP %d\n", result->httpstatus);
-		source->root->available = FALSE;
+	if (!(result->data && result->httpstatus == 200)) {
+		ttrss_source_set_login_error (source, g_strdup_printf ("Login request failed with HTTP error %d", result->httpstatus));
+		return;
 	}
 
+	parser = json_parser_new ();
+	if (!json_parser_load_from_data (parser, result->data, -1, NULL)) {
+		ttrss_source_set_login_error (source, g_strdup ("Invalid JSON returned on login!"));
+		return;
+	}
+	
+	JsonNode *node = json_parser_get_root (parser);
+
+	/* Check for API specified error... */
+	if (json_get_string (json_get_node (node, "content"), "error")) {
+		const gchar *error = json_get_string (json_get_node (node, "content"), "error");
+
+		ttrss_source_set_login_error (source, g_strdup (error));
+		if (g_str_equal (error, "LOGIN_ERROR"))
+			auth_dialog_new (source->root->subscription, flags);
+
+		return;
+	}
+
+	/* Success! Get SID and API Level. */
+	source->apiLevel = json_get_int (json_get_node (node, "content"), "api_level");
+	source->session_id = g_strdup (json_get_string (json_get_node (node, "content"), "session_id"));
+	if (source->session_id) {
+		debug2 (DEBUG_UPDATE, "TinyTinyRSS Found session_id: >>>%s<<< (API level %d)!", source->session_id, source->apiLevel);
+	} else {
+		ttrss_source_set_login_error (source, g_strdup_printf ("No session_id found in response!\n%s", result->data));
+	}
+	
+	g_object_unref (parser);
+
 	if (source->session_id) {
 		updateRequestPtr request;
+		gchar *source_uri;
 
 		/* Check for remote update daemon running. This needs to be known
-		   before we start updating to decide wether to actively update
+		   before we start updating to decide whether to actively update
 		   remote feeds or just fetch them. */
 		request = update_request_new ();
 		request->options = update_options_copy (subscription->updateOptions);
 		request->postdata = g_strdup_printf (TTRSS_JSON_GET_CONFIG, source->session_id);
 
-		update_request_set_source (request, g_strdup_printf (TTRSS_URL, metadata_list_get (subscription->metadata, "ttrss-url")));
+		source_uri = g_strdup_printf (TTRSS_URL, source->url);
+		update_request_set_source (request, source_uri);
+		g_free (source_uri);
 		update_execute_request (source, request, ttrss_source_get_config_cb, source, flags);
 	}
 }
@@ -165,13 +191,13 @@ ttrss_source_login_cb (const struct updateResult * const result, gpointer userda
 void
 ttrss_source_login (ttrssSourcePtr source, guint32 flags) 
 { 
-	gchar			*username, *password;
+	gchar			*username, *password, *source_uri;
 	updateRequestPtr	request;
 	subscriptionPtr		subscription = source->root->subscription;
 	
 	if (source->loginState != TTRSS_SOURCE_STATE_NONE) {
 		/* this should not happen, as of now, we assume the session doesn't expire. */
-		debug1(DEBUG_UPDATE, "Logging in while login state is %d", source->loginState);
+		debug1 (DEBUG_UPDATE, "Logging in while login state is %d", source->loginState);
 	}
 
 	request = update_request_new ();
@@ -179,8 +205,16 @@ ttrss_source_login (ttrssSourcePtr source, guint32 flags)
 	/* escape user and password for JSON call */
 	username = g_strescape (subscription->updateOptions->username, NULL);
 	password = g_strescape (subscription->updateOptions->password, NULL);
-	
-	update_request_set_source (request, g_strdup_printf (TTRSS_URL, metadata_list_get (subscription->metadata, "ttrss-url")));
+
+	source->url = metadata_list_get (subscription->metadata, "ttrss-url");
+	if (!source->url) {
+		ttrss_source_set_login_error (source, g_strdup ("Fatal: We've lost the TinyTinyRSS server URL! Please re-subscribe!"));
+		return;
+	}
+
+	source_uri = g_strdup_printf (TTRSS_URL, source->url);
+	update_request_set_source (request, source_uri);
+	g_free (source_uri);
 
 	request->options = update_options_copy (subscription->updateOptions);
 	request->postdata = g_strdup_printf (TTRSS_JSON_LOGIN, username, password);
@@ -195,7 +229,6 @@ ttrss_source_login (ttrssSourcePtr source, guint32 flags)
 
 /* node source type implementation */
 
-
 static void
 ttrss_source_update (nodePtr node)
 {
@@ -224,18 +257,31 @@ ttrss_source_init (void)
 
 static void ttrss_source_deinit (void) { }
 
+static void
+ttrss_source_set_subscription_type (nodePtr folder)
+{
+	GSList *iter;
+
+	for (iter = folder->children; iter; iter = g_slist_next(iter)) {
+		nodePtr node = (nodePtr) iter->data;
+
+		if (node->subscription)
+			node->subscription->type = &ttrssSourceFeedSubscriptionType;
+		else
+			ttrss_source_set_subscription_type (node);
+	}
+}
+
 static void
 ttrss_source_import (nodePtr node)
 {
-	GSList *iter; 
 	opml_source_import (node);
-	
+
 	node->subscription->type = &ttrssSourceSubscriptionType;
 	if (!node->data)
 		node->data = (gpointer) ttrss_source_new (node);
 
-	for (iter = node->children; iter; iter = g_slist_next(iter))
-		((nodePtr) iter->data)->subscription->type = &ttrssSourceFeedSubscriptionType;
+	ttrss_source_set_subscription_type (node);
 }
 
 static void
@@ -267,23 +313,26 @@ on_ttrss_source_selected (GtkDialog *dialog,
 		nodePtr		node;
 		subscriptionPtr subscription = subscription_new ("", NULL, NULL);
 		
-		/* The is a bit ugly: we need to prevent the tt-rss base
+		/* This is a bit ugly: we need to prevent the tt-rss base
 		   URL from being lost by unwanted permanent redirects on
 		   the getFeeds call, so we save it as the homepage meta
 		   data value... */
 		metadata_list_set (&subscription->metadata, "ttrss-url", gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET (dialog), "serverUrlEntry"))));
-		
-		subscription->updateOptions->username = g_strdup (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "userEntry"))));
-		subscription->updateOptions->password = g_strdup (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "passwordEntry"))));
-		subscription->type = &ttrssSourceSubscriptionType;
-		
+
 		node = node_new (node_source_get_node_type ());
 		node_set_title (node, "Tiny Tiny RSS");
 		node_source_new (node, ttrss_source_get_type ());
 		node_set_subscription (node, subscription);
-		node->data = ttrss_source_new (node);
+		
+		subscription_set_auth_info (subscription,
+		                            gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "userEntry"))),
+		                            gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "passwordEntry"))));
+		subscription->type = &ttrssSourceSubscriptionType;		
+
+		node->data = (gpointer)ttrss_source_new (node);
 		feedlist_node_added (node);
 		ttrss_source_update (node);
+		db_node_update (node);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -306,6 +355,8 @@ static void
 ttrss_source_cleanup (nodePtr node)
 {
 	ttrssSourcePtr source = (ttrssSourcePtr) node->data;
+	g_hash_table_destroy (source->categories);
+	g_hash_table_destroy (source->categoryNodes);
 	ttrss_source_free (source);
 	node->data = NULL;
 }
@@ -313,7 +364,7 @@ ttrss_source_cleanup (nodePtr node)
 static void
 ttrss_source_remote_update_cb (const struct updateResult * const result, gpointer userdata, updateFlags flags)
 {
-	debug2 (DEBUG_UPDATE, "tt-rss result processing... status:%d >>>%s<<<", result->httpstatus, result->data);
+	debug2 (DEBUG_UPDATE, "TinyTinyRSS result processing... status:%d >>>%s<<<", result->httpstatus, result->data);
 }
 
 /* FIXME: Only simple synchronous item change requests... Get async! */
@@ -329,7 +380,7 @@ ttrss_source_item_set_flag (nodePtr node, itemPtr item, gboolean newStatus)
 	request->options = update_options_copy (root->subscription->updateOptions);
 	request->postdata = g_strdup_printf (TTRSS_JSON_UPDATE_ITEM_FLAG, source->session_id, item_get_id(item), newStatus?1:0 );
 
-	update_request_set_source (request, g_strdup_printf (TTRSS_URL, metadata_list_get (root->subscription->metadata, "ttrss-url")));
+	update_request_set_source (request, g_strdup_printf (TTRSS_URL, source->url));
 	update_execute_request (source, request, ttrss_source_remote_update_cb, source, 0 /* flags */);
 
 	item_flag_state_changed (item, newStatus);
@@ -346,7 +397,7 @@ ttrss_source_item_mark_read (nodePtr node, itemPtr item, gboolean newStatus)
 	request->options = update_options_copy (root->subscription->updateOptions);
 	request->postdata = g_strdup_printf (TTRSS_JSON_UPDATE_ITEM_UNREAD, source->session_id, item_get_id(item), newStatus?0:1 );
 
-	update_request_set_source (request, g_strdup_printf (TTRSS_URL, metadata_list_get (root->subscription->metadata, "ttrss-url")));
+	update_request_set_source (request, g_strdup_printf (TTRSS_URL, source->url));
 	update_execute_request (source, request, ttrss_source_remote_update_cb, source, 0 /* flags */);
 
 	item_read_state_changed (item, newStatus);
