@@ -1,7 +1,7 @@
 /**
  * @file itemset.c handling sets of items
  * 
- * Copyright (C) 2005-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2011 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2005-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -246,7 +246,7 @@ itemset_merge_item (itemSetPtr itemSet, GList *items, itemPtr item, gint maxChec
 			while (iter) {
 				enclosurePtr enc = enclosure_from_string (iter->data);
 				debug1 (DEBUG_UPDATE, "download enclosure (%s)", (gchar *)iter->data);
-				enclosure_save_as_file (NULL, enc->url, NULL);
+				enclosure_download (NULL, enc->url, FALSE /* non interactive */);
 				iter = g_slist_next (iter);
 				enclosure_free (enc);
 			}
@@ -454,12 +454,6 @@ itemset_check_item (itemSetPtr itemSet, itemPtr item)
 	return result;
 }
 
-gboolean
-itemset_has_item_id (itemSetPtr itemSet, gulong id)
-{
-	return (NULL != g_list_find (itemSet->ids, GUINT_TO_POINTER (id)));
-}
-
 void
 itemset_add_rule (itemSetPtr itemSet,
                   const gchar *ruleId,
