@@ -1,8 +1,8 @@
-/**
- * @file ui_shell.c  UI layout handling
+/*
+ * @file liferea_shell.c  UI layout handling
  *
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2007-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,27 +27,33 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <libpeas/peas-extension-set.h>
 
 #include "browser.h"
 #include "common.h"
 #include "conf.h"
+#include "db.h"
 #include "debug.h"
 #include "export.h"
 #include "feedlist.h"
 #include "htmlview.h"
+#include "item_history.h"
 #include "itemlist.h"
 #include "net_monitor.h"
+#include "plugins_engine.h"
+#include "render.h"
 #include "vfolder.h"
 #include "ui/browser_tabs.h"
+#include "ui/feed_list_node.h"
 #include "ui/feed_list_view.h"
 #include "ui/icons.h"
 #include "ui/itemview.h"
 #include "ui/item_list_view.h"
 #include "ui/liferea_dialog.h"
+#include "ui/liferea_shell_activatable.h"
+#include "ui/preferences_dialog.h"
 #include "ui/search_dialog.h"
 #include "ui/ui_common.h"
-#include "ui/ui_prefs.h"
-#include "ui/ui_session.h"
 #include "ui/ui_tray.h"
 #include "ui/ui_update.h"
 
@@ -78,6 +84,10 @@ struct LifereaShellPrivate {
 	FeedList	*feedlist;
 	ItemView	*itemview;
 	BrowserTabs	*tabs;
+
+	PeasExtensionSet *extensions;		/**< Plugin management */
+
+	gboolean	fullscreen;		/**< track fullscreen */
 };
 
 static GObjectClass *parent_class = NULL;
@@ -137,7 +147,7 @@ liferea_shell_init (LifereaShell *ls)
 	gtk_builder_connect_signals (shell->priv->xml, NULL);
 }
 
-/**
+/*
  * Restore the window position from the values saved into gconf. Note
  * that this does not display/present/show the mainwindow.
  */
@@ -186,7 +196,7 @@ liferea_shell_restore_position (void)
 
 }
 
-void
+static void
 liferea_shell_save_position (void)
 {
 	GtkWidget	*pane;
@@ -295,6 +305,13 @@ liferea_shell_update_allitems_actions (gboolean isNotEmpty, gboolean isRead)
 	gtk_action_set_sensitive (gtk_action_group_get_action (shell->priv->feedActions, "MarkFeedAsRead"), isRead);
 }
 
+void
+liferea_shell_update_history_actions (void)
+{
+	gtk_action_set_sensitive (gtk_action_group_get_action (shell->priv->generalActions, "PrevReadItem"), item_history_has_previous ());
+	gtk_action_set_sensitive (gtk_action_group_get_action (shell->priv->generalActions, "NextReadItem"), item_history_has_next ());
+}
+
 void
 liferea_shell_update_unread_stats (void)
 {
@@ -503,12 +520,18 @@ on_window_state_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 				conf_set_bool_value (LAST_WINDOW_MAXIMIZED, FALSE);
 		}
 		if (state & GDK_WINDOW_STATE_ICONIFIED)
-			session_set_cmd (NULL, MAINWINDOW_ICONIFIED);
+			conf_set_int_value (LAST_WINDOW_STATE, MAINWINDOW_ICONIFIED);
 		else if(state & GDK_WINDOW_STATE_WITHDRAWN)
-			session_set_cmd (NULL, MAINWINDOW_HIDDEN);
+			conf_set_int_value (LAST_WINDOW_STATE, MAINWINDOW_HIDDEN);
 		else
-			session_set_cmd (NULL, MAINWINDOW_SHOWN);
+			conf_set_int_value (LAST_WINDOW_STATE, MAINWINDOW_SHOWN);
 	}
+
+	if ((event->window_state.new_window_state & GDK_WINDOW_STATE_FULLSCREEN) == 0)
+		shell->priv->fullscreen = TRUE;
+	else
+		shell->priv->fullscreen = FALSE;
+
 	return FALSE;
 }
 
@@ -526,7 +549,7 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 
 		/* handle [<modifier>+]<Space> headline skimming hotkey */
 		switch (event->keyval) {
-			case GDK_space:
+			case GDK_KEY_space:
 				conf_get_int_value (BROWSE_KEY_SETTING, &browse_key_setting);
 				switch (browse_key_setting) {
 					default:
@@ -558,14 +581,14 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		/* some <Ctrl> hotkeys that overrule the HTML view */
 		if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK) {
 			switch (event->keyval) {
-				case GDK_KP_Add:
-				case GDK_equal:
-				case GDK_plus:
+				case GDK_KEY_KP_Add:
+				case GDK_KEY_equal:
+				case GDK_KEY_plus:
 					liferea_shell_do_zoom (TRUE);
 					return TRUE;
 					break;
-				case GDK_KP_Subtract:
-				case GDK_minus:
+				case GDK_KEY_KP_Subtract:
+				case GDK_KEY_minus:
 					liferea_shell_do_zoom (FALSE);
 					return TRUE;
 					break;
@@ -585,32 +608,32 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		/* check for treeview navigation */
 		if (0 == (event->state & default_modifiers)) {
 			switch (event->keyval) {
-				case GDK_KP_Delete:
-				case GDK_Delete:
+				case GDK_KEY_KP_Delete:
+				case GDK_KEY_Delete:
 					if (focusw == GTK_WIDGET (shell->priv->feedlistView))
 						return FALSE;	/* to be handled in feed_list_view_key_press_cb() */
 						
 					on_remove_item_activate (NULL, NULL);
 					return TRUE;
 					break;
-				case GDK_n: 
+				case GDK_KEY_n:
 					on_next_unread_item_activate (NULL, NULL);
 					return TRUE;
 					break;
-				case GDK_f:
+				case GDK_KEY_f:
 					itemview_move_cursor (1);
 					return TRUE;
 					break;
-				case GDK_b:
+				case GDK_KEY_b:
 					itemview_move_cursor (-1);
 					return TRUE;
 					break;
-				case GDK_u:
+				case GDK_KEY_u:
 					ui_common_treeview_move_cursor (shell->priv->feedlistView, -1);
 					itemview_move_cursor_to_first ();
 					return TRUE;
 					break;
-				case GDK_d:
+				case GDK_KEY_d:
 					ui_common_treeview_move_cursor (shell->priv->feedlistView, 1);
 					itemview_move_cursor_to_first ();
 					return TRUE;
@@ -627,7 +650,7 @@ liferea_shell_save_accels (gpointer data)
 {
 	gchar *accels_file = NULL;
 
-	accels_file = g_build_filename (common_get_cache_path (), "accels", NULL);
+	accels_file = common_create_config_filename ("accels");
 	gtk_accel_map_save (accels_file);
 	g_free (accels_file);
 	return FALSE;
@@ -641,13 +664,19 @@ on_accel_change (GtkAccelMap *object, gchar *accel_path,
 	g_idle_add (liferea_shell_save_accels, NULL);
 }
 
+static void
+on_prefbtn_clicked (GtkButton *button, gpointer user_data)
+{
+	preferences_dialog_open ();
+}
+
 static void
 on_searchbtn_clicked (GtkButton *button, gpointer user_data)
 {
 	simple_search_dialog_open ();
 }
 
-void
+static void
 on_onlinebtn_clicked (GtkButton *button, gpointer user_data)
 {
 	network_monitor_set_online (!network_monitor_is_online ());
@@ -708,6 +737,14 @@ on_menu_quit (GtkMenuItem *menuitem, gpointer user_data)
 	liferea_shutdown ();
 }
 
+static void
+on_menu_fullscreen_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	shell->priv->fullscreen == TRUE ?
+		gtk_window_fullscreen(shell->priv->window) :
+		gtk_window_unfullscreen (shell->priv->window);
+}
+
 static void
 on_menu_zoomin_selected (gpointer callback_data, guint callback_action, GtkWidget *widget)
 {
@@ -732,6 +769,24 @@ on_menu_export (gpointer callback_data, guint callback_action, GtkWidget *widget
 	export_OPML_file ();
 }
 
+static void
+on_extension_added (PeasExtensionSet *extensions,
+                    PeasPluginInfo   *info,
+                    PeasExtension    *exten,
+                    LifereaShell     *shell)
+{
+	peas_extension_call (exten, "activate");
+}
+
+static void
+on_extension_removed (PeasExtensionSet *extensions,
+                      PeasPluginInfo   *info,
+                      PeasExtension    *exten,
+                      LifereaShell     *shell)
+{
+	peas_extension_call (exten, "deactivate");
+}
+
 static void
 liferea_shell_set_online_icon (gboolean online)
 {
@@ -856,9 +911,14 @@ static const GtkActionEntry liferea_shell_action_entries[] = {
 	 G_CALLBACK(on_remove_items_activate)},
 
 	{"ItemMenu", NULL, N_("_Item")},
+	{"PrevReadItem", GTK_STOCK_GO_BACK, N_("Previous Item"), "<control><shift>N", NULL,	
+	 G_CALLBACK(on_prev_read_item_activate)},
+	{"NextReadItem", GTK_STOCK_GO_FORWARD, N_("Next Item"), NULL, NULL,	
+	 G_CALLBACK(on_next_read_item_activate)},
+
 	/* No tooltip here as it really hinders usability to have it flashing
 	   when skimming through items using "Next Unread"! */
-	{"NextUnreadItem", GTK_STOCK_GO_FORWARD, N_("_Next Unread Item"), "<control>N", NULL,	
+	{"NextUnreadItem", GTK_STOCK_JUMP_TO, N_("_Next Unread Item"), "<control>N", NULL,	
 	 G_CALLBACK(on_next_unread_item_activate)},
 
 	{"ViewMenu", NULL, N_("_View")},
@@ -926,13 +986,19 @@ static const GtkActionEntry liferea_shell_item_action_entries[] = {
 	 G_CALLBACK(on_toggle_item_flag)},
 	{"RemoveSelectedItem", "gtk-delete", N_("R_emove"), NULL, N_("Removes the selected item."),
 	 G_CALLBACK(on_remove_item_activate)},
-	{"LaunchItemInBrowser", NULL, N_("_Launch In Browser"), NULL, N_("Launches the item's link in the configured browser."),
-	 G_CALLBACK(on_popup_launchitem_selected)}
+	{"LaunchItemInTab", NULL, N_("Open In _Tab"), NULL, N_("Launches the item's link in a new Liferea browser tab."),
+	 G_CALLBACK(on_popup_launch_item_in_tab_selected)},
+	{"LaunchItemInBrowser", NULL, N_("_Open In Browser"), NULL, N_("Launches the item's link in the Liferea item pane."),
+	 G_CALLBACK(on_popup_launch_item_selected)},
+	{"LaunchItemInExternalBrowser", NULL, N_("Open In _External Browser"), NULL, N_("Launches the item's link in the configured external browser."),
+	 G_CALLBACK(on_popup_launch_item_selected)}
 };
 
 static const GtkToggleActionEntry liferea_shell_action_toggle_entries[] = {
 	{"ToggleOfflineMode", NULL, N_("_Work Offline"), NULL, N_("This option allows you to disable subscription updating."),
-	 G_CALLBACK(on_work_offline_activate)}
+	 G_CALLBACK(on_work_offline_activate)},
+	{"FullScreen", NULL, N_("_Fullscreen"), "F11", N_("Browse at full screen"),
+	 G_CALLBACK(on_menu_fullscreen_activate)},
 };
 
 static const char *liferea_shell_ui_desc =
@@ -967,13 +1033,20 @@ static const char *liferea_shell_ui_desc =
 "    <menu action='ItemMenu'>"
 "      <menuitem action='NextUnreadItem'/>"
 "      <separator/>"
+"      <menuitem action='PrevReadItem'/>"
+"      <menuitem action='NextReadItem'/>"
+"      <separator/>"
 "      <menuitem action='ToggleItemReadStatus'/>"
 "      <menuitem action='ToggleItemFlag'/>"
 "      <menuitem action='RemoveSelectedItem'/>"
 "      <separator/>"
+"      <menuitem action='LaunchItemInTab'/>"
 "      <menuitem action='LaunchItemInBrowser'/>"
+"      <menuitem action='LaunchItemInExternalBrowser'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
+"      <menuitem action='FullScreen'/>"
+"      <separator/>"
 "      <menuitem action='ZoomIn'/>"
 "      <menuitem action='ZoomOut'/>"
 "      <separator/>"
@@ -1001,8 +1074,10 @@ static const char *liferea_shell_ui_desc =
 "  <toolbar action='maintoolbar'>"
 "    <separator/>"
 "    <toolitem name='newFeedButton' action='NewSubscription'/>"
-"    <toolitem name='nextUnreadButton' action='NextUnreadItem'/>"
 "    <toolitem name='MarkAsReadButton' action='MarkFeedAsRead'/>"
+"    <toolitem name='prevReadButton' action='PrevReadItem'/>"
+"    <toolitem name='nextReadButton' action='NextReadItem'/>"
+"    <toolitem name='nextUnreadButton' action='NextUnreadItem'/>"
 "    <toolitem name='UpdateAllButton' action='UpdateAll'/>"
 "    <toolitem name='SearchButton' action='SearchFeeds'/>"
 "    <separator/>"
@@ -1010,10 +1085,13 @@ static const char *liferea_shell_ui_desc =
 "</ui>";
 
 static void
-liferea_shell_restore_state (void)
+liferea_shell_restore_state (const gchar *overrideWindowState)
 {
-	gchar *toolbar_style, *accels_file;
-	gint last_vpane_pos, last_hpane_pos, last_wpane_pos;
+	gchar		*toolbar_style, *accels_file;
+	gint		last_vpane_pos, last_hpane_pos, last_wpane_pos;
+	gint		resultState;
+	gboolean	show_tray_icon, start_in_tray;
+
 	
 	debug0 (DEBUG_GUI, "Setting toolbar style");
 	
@@ -1023,7 +1101,7 @@ liferea_shell_restore_state (void)
 
 	debug0 (DEBUG_GUI, "Loading accelerators");
 	
-	accels_file = g_build_filename (common_get_cache_path(), "accels", NULL);
+	accels_file = common_create_config_filename ("accels");
 	gtk_accel_map_load (accels_file);
 	g_free (accels_file);	
 
@@ -1042,26 +1120,81 @@ liferea_shell_restore_state (void)
 	conf_get_int_value (LAST_WPANE_POS, &last_wpane_pos);
 	if (last_wpane_pos)
 		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), last_wpane_pos);
+
+	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
+	conf_get_bool_value (START_IN_TRAY, &start_in_tray);
+
+	/* Apply horrible window state parameter logic:
+	   -> overrideWindowState provides optional command line flags passed by
+	      user or the session manager (prio 1)
+	   -> lastState provides last shutdown preference (prio 2)
+	   -> show_tray_icon and start_in_tray are overriding preferences
+	      (prio 1 only overridden by 'shown' from command line)
+	 */
+
+	/* Initialize with last saved state */
+	conf_get_int_value (LAST_WINDOW_STATE, &resultState);
+
+	/* Override with command line options */
+	if (!g_strcmp0 (overrideWindowState, "iconfied"))
+		resultState = MAINWINDOW_ICONIFIED;
+	if (!g_strcmp0 (overrideWindowState, "hidden"))
+		resultState = MAINWINDOW_HIDDEN;
+	if (!g_strcmp0 (overrideWindowState, "shown"))
+		resultState = MAINWINDOW_SHOWN;
+
+	/* Change hidden to iconify if we have no tray to avoid loosing the window */
+	if (!show_tray_icon && resultState == MAINWINDOW_HIDDEN)
+		resultState = MAINWINDOW_ICONIFIED;
+
+	/* Apply tray hiding preference */
+	if (show_tray_icon && start_in_tray && g_strcmp0 (overrideWindowState, "shown")) {
+		debug0 (DEBUG_GUI, "Ignoring last window state due to hide-in-tray preference");
+		resultState = MAINWINDOW_HIDDEN;
+	}
+
+	/* And set the window to the resulting state */
+	switch (resultState) {
+		case MAINWINDOW_ICONIFIED:
+			debug0 (DEBUG_GUI, "Restoring window state 'hidden (no tray)' / 'iconified'");
+			gtk_window_iconify (shell->priv->window);
+			gtk_widget_show (GTK_WIDGET (shell->priv->window));
+			break;
+		case MAINWINDOW_HIDDEN:
+			debug0 (DEBUG_GUI, "Restoring window state 'hidden (to tray)'");
+			/* Realize needed so that the window structure can be
+			   accessed... otherwise we get a GTK warning when window is
+			   shown by clicking on notification icon or when theme
+			   colors are fetched. */
+			gtk_widget_realize (GTK_WIDGET (shell->priv->window));
+			gtk_widget_hide (GTK_WIDGET (shell->priv->window));
+			break;
+		case MAINWINDOW_SHOWN:
+		default:
+			/* Safe default is always to show window */
+			debug0 (DEBUG_GUI, "Restoring window state 'shown'");
+			gtk_widget_show (GTK_WIDGET (shell->priv->window));
+	}
 }
 
 void
-liferea_shell_create (int initialState)
+liferea_shell_create (GtkApplication *app, const gchar *overrideWindowState)
 {
 	GtkUIManager	*ui_manager;
 	GtkAccelGroup	*accel_group;
 	GError		*error = NULL;	
-	GString		*buffer;
 	gboolean	show_tray_icon, toggle;
+	gchar		*id;
 	
 	debug_enter ("liferea_shell_create");
 
-	/* 1.) object setup */
-
 	g_object_new (LIFEREA_SHELL_TYPE, NULL);
 
 	shell->priv->window = GTK_WINDOW (liferea_shell_lookup ("mainwindow"));
+
+	gtk_window_set_application (GTK_WINDOW (shell->priv->window), app);
 	
-	/* 2.) menu creation */
+	/* 1.) menu creation */
 	
 	debug0 (DEBUG_GUI, "Setting up menues");
 
@@ -1113,6 +1246,9 @@ liferea_shell_create (int initialState)
 	shell->priv->menubar = gtk_ui_manager_get_widget (ui_manager, "/MainwindowMenubar");
 	shell->priv->toolbar = gtk_ui_manager_get_widget (ui_manager, "/maintoolbar");
 
+	/* Ensure GTK3 toolbar shadows... */
+	gtk_style_context_add_class (gtk_widget_get_style_context (shell->priv->toolbar), "primary-toolbar");
+
 	/* what a pain, why is there no markup for this option? */
 	g_object_set (G_OBJECT (gtk_ui_manager_get_widget (ui_manager, "/maintoolbar/newFeedButton")), "is_important", TRUE, NULL);
 	g_object_set (G_OBJECT (gtk_ui_manager_get_widget (ui_manager, "/maintoolbar/nextUnreadButton")), "is_important", TRUE, NULL);
@@ -1154,6 +1290,9 @@ liferea_shell_create (int initialState)
 	shell->priv->statusbar_feedsinfo = gtk_label_new("");
 	gtk_widget_show(shell->priv->statusbar_feedsinfo);
 	gtk_box_pack_start (GTK_BOX (shell->priv->statusbar), shell->priv->statusbar_feedsinfo, FALSE, FALSE, 5);
+
+	g_signal_connect ((gpointer) liferea_shell_lookup ("onlinebtn"), "clicked",
+	                  G_CALLBACK (on_onlinebtn_clicked), NULL);
 	
 	/* 4.) setup tabs */
 	
@@ -1189,85 +1328,60 @@ liferea_shell_create (int initialState)
 	
 	icons_load ();
 	
-	/* 9.) update all menu elements */
-	
+	/* 9.) update and restore all menu elements */
+
 	liferea_shell_update_toolbar ();
+	liferea_shell_update_history_actions ();
+	liferea_shell_setup_URL_receiver ();
+	liferea_shell_restore_state (overrideWindowState);
 	
-	liferea_shell_setup_URL_receiver ();	/* setup URL dropping support */
+	gtk_widget_set_sensitive (GTK_WIDGET (shell->priv->feedlistView), TRUE);
+
+	/* 10.) After main window is realized get theme colors and set up feed
+ 	        list and tray icon */
+	render_init_theme_colors (GTK_WIDGET (shell->priv->window));
 
 	shell->priv->feedlist = feedlist_create ();
 
 	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
+	ui_tray_enable (show_tray_icon);
 
-	ui_tray_enable (show_tray_icon);		/* init tray icon */
+	/* 11.) Restore latest selection */
 
-	liferea_shell_restore_state ();
-	
-	if (initialState == MAINWINDOW_ICONIFIED || 
-	    (initialState == MAINWINDOW_HIDDEN && ui_tray_get_count () == 0)) {
-		gtk_window_iconify (shell->priv->window);
-		gtk_widget_show (GTK_WIDGET (shell->priv->window));
-	} else if (initialState == MAINWINDOW_SHOWN) {
-		gtk_widget_show (GTK_WIDGET (shell->priv->window));
-	} else {
-		/* Needed so that the window structure can be
-		   accessed... otherwise will GTK warning when window is
-		   shown by clicking on notification icon. */
-		   
-		/* Note: gtk_widget_realize () did not work with GtkMozEmbed
-		   therefore we use show+hide instead. */
-		gtk_widget_show (GTK_WIDGET (shell->priv->window));
-		gtk_widget_hide (GTK_WIDGET (shell->priv->window));
+	// FIXME: Move to feed list code
+	if (conf_get_str_value (LAST_NODE_SELECTED, &id)) {
+		feed_list_view_select (node_from_id (id));
+		g_free (id);
+
+		// FIXME: Move to item list view code
+		gint item_id;
+		if (conf_get_int_value (LAST_ITEM_SELECTED, &item_id)) {
+			itemPtr item = db_item_load ((gulong)item_id);
+			if (item) {
+				itemview_select_item (item);
+				item_unload (item);
+			}
+		}
 	}
-
-	gtk_widget_set_sensitive (GTK_WIDGET (shell->priv->feedlistView), TRUE);
-	
-	/* 9. Create welcome text */
-
-	/* force two pane mode */
-	/*   For some reason, this causes the first item to be selected and then
-	     unselected... strange. */
-	feed_list_view_select (NULL);
-	
-	itemview_set_layout (NODE_VIEW_MODE_COMBINED);
-	
-	buffer = g_string_new (NULL);
-	htmlview_start_output (buffer, NULL, TRUE, FALSE);
-	g_string_append (buffer,   "<div style=\"padding:8px\">"
-				   "<table class=\"headmeta\" style=\"border:solid 1px #aaa;font-size:120%\" border=\"0\" cellspacing=\"0\" cellpadding=\"5px\"><tr><td>"
-				   // Display application icon
-				   "<img src=\""
-				   PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S
-				   "hicolor" G_DIR_SEPARATOR_S "48x48" G_DIR_SEPARATOR_S "apps"
-				   G_DIR_SEPARATOR_S "liferea.png\" />"
-				   "</td><td><h3>");
-	g_string_append (buffer,   _("Liferea - Linux Feed Reader"));
-	g_string_append (buffer,   "</h3></td></tr><tr><td colspan=\"2\">");
-	g_string_append (buffer,   _("<p>Welcome to <b>Liferea</b>, a desktop news aggregator for online news "
-				   "feeds.</p>"
-				   "<p>You can add new subscriptions "
-	                           "<ul>"
-	                           "<li>From main menu 'Subscription' -&gt; 'New Subscription'</li>"
-	                           "<li>By dropping feed links into the subscription list</li>"
-	                           "<li>By right clicking links and choosing 'Subscribe' within Liferea</li>"
-	                           "</ul>"
-				   "</p>"));
-	g_string_append (buffer,   "</td></tr></table>");
-	
-	g_string_append (buffer,   "</div>");
-
-	htmlview_finish_output (buffer);
-	itemview_display_info (buffer->str);
-	g_string_free (buffer, TRUE);
-	
-	/* 10. Connect network monitoring and set icon*/
+		
+	/* 12. Connect network monitoring and set icon*/
 	
 	g_signal_connect (network_monitor_get (), "online-status-changed",
 	                  G_CALLBACK (liferea_shell_online_status_changed), shell);
 
 	liferea_shell_set_online_icon (network_monitor_is_online ());
 
-	/* 11. Rebuild search folders if needed */
+	/* 13. Setup shell plugins */
+
+	shell->priv->extensions = peas_extension_set_new (PEAS_ENGINE (liferea_plugins_engine_get_default ()),
+		                             LIFEREA_TYPE_SHELL_ACTIVATABLE, "shell", shell, NULL);
+
+	g_signal_connect (shell->priv->extensions, "extension-added", G_CALLBACK (on_extension_added), shell);
+	g_signal_connect (shell->priv->extensions, "extension-removed",	G_CALLBACK (on_extension_removed), shell);
+
+	peas_extension_set_call (shell->priv->extensions, "activate");
+
+	/* 14. Rebuild search folders if needed */
 	if (searchFolderRebuild)
 		vfolder_foreach (vfolder_rebuild);
 
@@ -1277,12 +1391,28 @@ liferea_shell_create (int initialState)
 void
 liferea_shell_destroy (void)
 {
-	feed_list_view_select (NULL);
+	nodePtr	node;
+	itemPtr	item;
+
+	/* Save last selection for next start */
+	// FIXME: Move to feed list handling
+	node = feedlist_get_selected ();
+	if (node)
+		conf_set_str_value (LAST_NODE_SELECTED, node->id);
+
+	// FIXME: Move to item list handling
+	item = itemlist_get_selected ();
+	if (item)
+		conf_set_int_value (LAST_ITEM_SELECTED, item->id);
+
+	feed_list_view_select (NULL);	// FIXME: Move to feed list handling
+
 	liferea_shell_save_position ();
 	ui_tray_enable (FALSE);
 	g_object_unref (shell->priv->tabs);
 	g_object_unref (shell->priv->feedlist);
 	g_object_unref (shell->priv->itemview);
+	g_object_unref (shell->priv->extensions);
 
 	gtk_widget_destroy (GTK_WIDGET (shell->priv->window));
 
