@@ -1,7 +1,7 @@
 /**
  * @file popup_menu.c popup menus
  *
- * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2013 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
@@ -28,18 +28,17 @@
 #include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
-#include "itemlist.h"
 #include "net_monitor.h"
 #include "newsbin.h"
 #include "node.h"
 #include "social.h"
 #include "vfolder.h"
 #include "ui/enclosure_list_view.h"
+#include "ui/feed_list_view.h"
 #include "ui/item_list_view.h"
 #include "ui/itemview.h"
 #include "ui/liferea_shell.h"
-#include "ui/feed_list_view.h"
-#include "ui/ui_prefs.h"
+#include "ui/preferences_dialog.h"
 #include "fl_sources/node_source.h"
 
 #define UI_POPUP_ITEM_IS_TOGGLE		1
@@ -56,6 +55,18 @@ on_toggle_visibility (void)
 	liferea_shell_toggle_visibility ();
 }
 
+static void
+on_popup_toggle_online (void)
+{
+	network_monitor_set_online (!network_monitor_is_online ());
+}
+
+static void
+on_popup_preferences (void)
+{
+	preferences_dialog_open ();
+}
+
 static void
 ui_popup_menu_at_pos (GtkWidget *menu, GtkMenuPositionFunc func, guint button, guint32 activate_time, gpointer user_data)
 {
@@ -113,10 +124,11 @@ ui_popup_item_menu (itemPtr item, guint button, guint32 activate_time)
 
 	menu = gtk_menu_new ();
 
-	ui_popup_add_menuitem (menu, _("Launch Item In _Tab"), on_popup_launchitem_in_tab_selected, NULL, NULL, 0);
-	ui_popup_add_menuitem (menu, _("_Launch Item In Browser"), on_popup_launchitem_selected, NULL, NULL, 0);
+	ui_popup_add_menuitem (menu, _("Open In _Tab"), on_popup_launch_item_in_tab_selected, NULL, NULL, 0);
+	ui_popup_add_menuitem (menu, _("_Open In Browser"), on_popup_launch_item_selected, NULL, NULL, 0);
+	ui_popup_add_menuitem (menu, _("Open In _External Browser"), on_popup_launch_item_external_selected, NULL, NULL, 0);
 
-	gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
 	iter = newsbin_get_list ();
 	if (iter) {
@@ -135,18 +147,18 @@ ui_popup_item_menu (itemPtr item, guint button, guint32 activate_time)
 			i++;
 		}
 
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM(item), submenu);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
 
-		gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 	}
 
-	text = g_strdup_printf (_("_Bookmark Link at %s"), social_get_bookmark_site ());
+	text = g_strdup_printf (_("_Bookmark at %s"), social_get_bookmark_site ());
 	ui_popup_add_menuitem (menu, text, on_popup_social_bm_item_selected, NULL, NULL, 0);
 	g_free (text);
 
-	ui_popup_add_menuitem (menu, _("Copy Item _URL to Clipboard"), on_popup_copy_URL_clipboard, NULL, NULL, 0);
+	ui_popup_add_menuitem (menu, _("Copy Item _Location"), on_popup_copy_URL_clipboard, NULL, NULL, 0);
 	
-	gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
 	ui_popup_add_menuitem (menu, _("Toggle _Read Status"), on_popup_toggle_read, NULL, GTK_STOCK_APPLY, 0);
 	ui_popup_add_menuitem (menu, _("Toggle Item _Flag"), on_popup_toggle_flag, NULL, NULL, 0);
@@ -178,11 +190,11 @@ ui_popup_systray_menu (GtkMenuPositionFunc func, guint button, guint32 activate_
 
 	menu = gtk_menu_new ();
 
-	ui_popup_add_menuitem (menu, _("_Work Offline"), on_onlinebtn_clicked, NULL, NULL, (!network_monitor_is_online ()) + UI_POPUP_ITEM_IS_TOGGLE);
+	ui_popup_add_menuitem (menu, _("_Work Offline"), on_popup_toggle_online, NULL, NULL, (!network_monitor_is_online ()) + UI_POPUP_ITEM_IS_TOGGLE);
 	ui_popup_add_menuitem (menu, _("_Update All"), on_menu_update_all, NULL, GTK_STOCK_REFRESH, 0);
-	ui_popup_add_menuitem (menu, NULL, on_prefbtn_clicked, NULL, GTK_STOCK_PREFERENCES, 0);
+	ui_popup_add_menuitem (menu, NULL, on_popup_preferences, NULL, GTK_STOCK_PREFERENCES, 0);
 
-	gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
 	ui_popup_add_menuitem (menu, _("_Show Liferea"), on_toggle_visibility, NULL, NULL, (!(gdk_window_get_state (gtk_widget_get_window (mainwindow)) & GDK_WINDOW_STATE_ICONIFIED) && gtk_widget_get_visible (mainwindow)) + UI_POPUP_ITEM_IS_TOGGLE);
 	ui_popup_add_menuitem (menu, NULL, on_popup_quit, NULL, GTK_STOCK_QUIT, 0);
@@ -268,20 +280,18 @@ static void
 ui_popup_node_menu (nodePtr node, gboolean validSelection, guint button, guint32 activate_time)
 {
 	GtkWidget	*menu;
-	gboolean	writeableFeedlist, isRoot, isHierarchic, addChildren;
+	gboolean	writeableFeedlist, isRoot, addChildren;
 
 	menu = gtk_menu_new ();
 	
 	if (node->parent) {
 		writeableFeedlist = NODE_SOURCE_TYPE (node->parent->source->root)->capabilities & NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST;
 		isRoot = NODE_SOURCE_TYPE (node->source->root)->capabilities & NODE_SOURCE_CAPABILITY_IS_ROOT;
-		isHierarchic = NODE_SOURCE_TYPE (node->parent->source->root)->capabilities & NODE_SOURCE_CAPABILITY_HIERARCHIC_FEEDLIST;
 		addChildren = NODE_TYPE (node->source->root)->capabilities & NODE_CAPABILITY_ADD_CHILDS;
 	} else {
 		/* if we have no parent then we have the root node... */
 		writeableFeedlist = TRUE;
 		isRoot = TRUE;
-		isHierarchic = TRUE;
 		addChildren = TRUE;
 	}
 
@@ -319,19 +329,19 @@ ui_popup_node_menu (nodePtr node, gboolean validSelection, guint button, guint32
 		}
 		
 		if (isRoot && node->children) {
-			gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 			ui_popup_add_menuitem (menu, _("Sort Feeds"), ui_popup_sort_feeds, node, GTK_STOCK_SORT_ASCENDING, 0);
 		}
 	}
 
 	if (IS_VFOLDER (node)) {
-		gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 		ui_popup_add_menuitem (menu, _("_Rebuild"), ui_popup_rebuild_vfolder, node, NULL, 0);
 	}
 
 	if (validSelection) {
 		if (writeableFeedlist) {
-			gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 			ui_popup_add_menuitem (menu, NULL, ui_popup_delete, node, GTK_STOCK_DELETE, 0);
 			ui_popup_add_menuitem (menu, NULL, ui_popup_properties, node, GTK_STOCK_PROPERTIES, 0);
 		}
